@@ -1,10 +1,9 @@
 from __future__ import annotations
 
 import abc
-
 import typing
 
-from lime_etl.domain import job_test_result, value_objects
+from lime_etl.domain import job_test_result, shared_resource, value_objects
 from lime_etl.services import job_logging_service, unit_of_work
 
 
@@ -19,6 +18,11 @@ class JobSpec(abc.ABC):
     def job_name(self) -> value_objects.JobName:
         raise NotImplementedError
 
+    @property
+    @abc.abstractmethod
+    def max_retries(self) -> value_objects.MaxRetries:
+        raise NotImplementedError
+
     @property
     @abc.abstractmethod
     def seconds_between_refreshes(self) -> value_objects.SecondsBetweenRefreshes:
@@ -29,11 +33,6 @@ class JobSpec(abc.ABC):
     def timeout_seconds(self) -> value_objects.TimeoutSeconds:
         raise NotImplementedError
 
-    @property
-    @abc.abstractmethod
-    def max_retries(self) -> value_objects.MaxRetries:
-        raise NotImplementedError
-
 
 class AdminJobSpec(JobSpec):
     @abc.abstractmethod
@@ -54,14 +53,25 @@ class AdminJobSpec(JobSpec):
 
 
 class ETLJobSpec(JobSpec):
+    @property
+    @abc.abstractmethod
+    def resources_needed(
+        self,
+    ) -> typing.Collection[value_objects.ResourceName]:
+        raise NotImplementedError
+
     @abc.abstractmethod
     def run(
-        self, logger: job_logging_service.JobLoggingService
+        self,
+        logger: job_logging_service.JobLoggingService,
+        resources: typing.Mapping[value_objects.ResourceName, typing.Any],
     ) -> value_objects.Result:
         raise NotImplementedError
 
     @abc.abstractmethod
     def test(
-        self, logger: job_logging_service.JobLoggingService
+        self,
+        logger: job_logging_service.JobLoggingService,
+        resources: typing.Mapping[value_objects.ResourceName, typing.Any],
     ) -> typing.Collection[job_test_result.SimpleJobTestResult]:
         raise NotImplementedError
